import logging
import platform
import socket
import subprocess
import sys
import uuid

from licensespring_hardware_id_generator import (
    HardwareIdAlgorithm,
    get_hardware_id,
    get_logs,
    get_version,
)


class HardwareIdProvider:
    def get_id(self):
        return str(uuid.getnode())

    def get_os_ver(self):
        return platform.platform()

    def get_hostname(self):
        return platform.node()

    def get_ip(self):
        return socket.gethostbyname(self.get_hostname())

    def get_is_vm(self):
        return False

    def get_vm_info(self):
        return None

    def get_mac_address(self):
        return ":".join(("%012X" % uuid.getnode())[i : i + 2] for i in range(0, 12, 2))

    def get_request_id(self):
        return str(uuid.uuid4())


class HardwareIdProviderSource(HardwareIdProvider):
    def get_id(self):
        hardware_id = get_hardware_id(HardwareIdAlgorithm.Default)

        logs = get_logs()
        version = get_version()
        logging.info("Version: ", version)
        logging.info("Hardware ID:", hardware_id)
        for log_line in logs:
            logging.info(log_line)

        return hardware_id


class PlatformIdProvider(HardwareIdProvider):

    def _read_from_win_registry(self, registry, key):
        try:
            from winregistry import WinRegistry

            with WinRegistry() as reg:
                return reg.read_entry(registry, key).value.strip()
        except Exception:
            return None

    def _execute_command(self, cmd):
        try:
            return subprocess.run(
                cmd, shell=True, capture_output=True, check=True, encoding="utf-8"
            ).stdout.strip()
        except Exception:
            return None

    def _read_from_file(self, path):
        try:
            with open(path) as f:
                return f.read().strip()
        except Exception:
            return None

    def _get_mac_id(self):
        return self._execute_command(
            "ioreg -d2 -c IOPlatformExpertDevice | awk -F\\\" '/IOPlatformUUID/{print $(NF-1)}'"
        )

    def _get_windows_id(self):
        id = self._read_from_win_registry(
            "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Cryptography", "MachineGuid"
        )
        if not id:
            result = self._execute_command("wmic csproduct get uuid")
            if result:
                lines = result.splitlines()
                if len(lines) >= 3:
                    id = lines[2].strip()
        return id

    def _get_linux_id(self):
        return self._read_from_file("/var/lib/dbus/machine-id") or self._read_from_file(
            "/etc/machine-id"
        )

    def _get_bsd_id(self):
        return self._read_from_file("/etc/hostid") or self._execute_command(
            "kenv -q smbios.system.uuid"
        )

    def get_id(self):
        platform = sys.platform
        id = None

        if platform == "darwin":
            id = self._get_mac_id()
        elif platform in ("win32", "cygwin", "msys"):
            id = self._get_windows_id()
        elif platform.startswith("linux"):
            id = self._get_linux_id()
        elif platform.startswith("openbsd") or platform.startswith("freebsd"):
            id = self._get_bsd_id()

        return id or super().get_id()



import copy
import json
import logging

from licensespring.licensefile import License
from licensespring.licensefile.base_manager import BaseManager
from licensespring.licensefile.config import Configuration
from licensespring.licensefile.data_storage import DataStorage
from licensespring.licensefile.license_manager import LicenseID
from licensespring.licensefile.offline_activation_guard import OfflineActivation


class BundleManager(BaseManager):
    def __init__(self, conf: Configuration):
        super().__init__(conf)
        self.cache_licenses = None

    def get_current_bundle(self) -> dict[str, License]:
        """
        Retrieve a dictionary of bundled licenses.

        Returns:
            dict: A dictionary where the keys are product short codes and the values are License objects.
        """
        if not self.cache_licenses:
            self.cache_licenses = self.load_bundles()
        return self.cache_licenses

    def load_bundles(self) -> dict[str, License]:
        """
        Load bundles from a file

        Returns:
            dict: A dictionary where the keys are product short codes and the values are License objects.
        """
        cache_licenses = {}

        bundle_response = self.licensefile_handler.get_licensefile_dict()

        for product_code, license_data in bundle_response.items():
            license_conf = copy.deepcopy(self._conf)
            license_conf._product = product_code
            license_storage = DataStorage(conf=license_conf)
            license_storage._cache.update_cache("normal", json.loads(license_data))
            cache_licenses[product_code] = License(
                product=product_code,
                api_client=self.api_client,
                licensefile_handler=license_storage,
            )

        return cache_licenses

    def activate_bundle(
        self,
        license_id: LicenseID,
        hardware_id: str = None,
        unique_license_id: int = None,
        customer_account_code: str = None,
        id_token: str = None,
        code: str = None,
        app_ver: str = None,
        os_ver: str = None,
        hostname: str = None,
        ip: str = None,
        is_vm: bool = None,
        vm_info: str = None,
        mac_address: str = None,
        variables: dict = None,
    ) -> dict[str, License]:
        """
        Activate bundle and update the licensefile

        Args:
            license_id (LicenseID): An instance containing the license key or user credentials.
            hardware_id (str, optional): A unique identifier for the hardware.
            unique_license_id (int, optional): A unique identifier for the license.
            customer_account_code (str, optional): An account code for the customer.
            id_token (str, optional): Token for identity verification.
            code (str, optional): An additional code for license verification.
            app_ver (str, optional): The version of the application requesting activation.
            os_ver (str, optional): The operating system version of the host.
            hostname (str, optional): The hostname of the device requesting activation.
            ip (str, optional): The IP address of the device.
            is_vm (bool, optional): Indicates whether the application is running on a virtual machine.
            vm_info (str, optional): Information about the virtual machine, if applicable.
            mac_address (str, optional): The MAC address of the device.

        Returns:
            dict[str, License]: A dictionary where the keys are product short codes and the values are License objects.
        """

        response = self.api_client.activate_bundle(
            product=self._conf._product,
            hardware_id=hardware_id,
            license_key=license_id.key,
            license_id=unique_license_id,
            username=license_id.username,
            password=license_id.password,
            customer_account_code=customer_account_code,
            id_token=id_token,
            code=code,
            app_ver=app_ver,
            os_ver=os_ver,
            hostname=hostname,
            ip=ip,
            is_vm=is_vm,
            vm_info=vm_info,
            mac_address=mac_address,
            variables=variables,
        )
        bundles_response = self.licensefile_handler._cache.get_bundle_response(response)

        response_licenses = {}

        for product_code, license_response in bundles_response.items():
            license_storage = DataStorage(self._conf)
            license_storage._cache.product = product_code
            license_storage._cache.update_cache(
                "activate_bundle_license", license_response
            )
            response_licenses[product_code] = License(
                product=product_code,
                api_client=self.api_client,
                licensefile_handler=license_storage,
            )
            license_obj = response_licenses[product_code]
            self.licensefile_handler.save_licensefile(
                license_obj.licensefile_handler._cache
            )

        self.cache_licenses = response_licenses

        return self.cache_licenses

    def create_offline_activation_file(
        self,
        license_id: LicenseID,
        req_path: str,
        hardware_id: str = None,
        unique_license_id: int = None,
        app_ver: str = None,
        os_ver: str = None,
        hostname: str = None,
        ip: str = None,
        is_vm: bool = None,
        vm_info: str = None,
        mac_address: str = None,
        device_variables: dict = None,
    ) -> str:
        """
        Creates .req file for activation

        Args:
            license_id (LicenseID): An instance containing the license key or user credentials.
            req_path (str): Specify place where to create .req file
            hardware_id (str, optional): A unique identifier for the hardware.
            app_ver (str, optional): The version of the application requesting activation.
            os_ver (str, optional): The operating system version of the host.
            hostname (str, optional): The hostname of the device requesting activation.
            ip (str, optional):  The IP address of the device.
            is_vm (bool, optional): Indicates whether the application is running on a virtual machine.
            vm_info (str, optional): Information about the virtual machine.
            mac_address (str, optional): The MAC address of the device.
            device_variables (dict, optional): device varaibles.

        Returns:
            str: path of the .req file
        """

        data = self.api_client.activate_offline_dump(
            product=self._conf._product,
            bundle_code=None,
            hardware_id=hardware_id,
            license_key=license_id.key,
            license_id=unique_license_id,
            username=license_id.username,
            password=license_id.password,
            app_ver=app_ver,
            os_ver=os_ver,
            hostname=hostname,
            ip=ip,
            is_vm=is_vm,
            vm_info=vm_info,
            mac_address=mac_address,
            variables=device_variables,
        )

        offline_data = OfflineActivation()
        offline_data.set_data(data)
        offline_data.set_is_activation(True)
        offline_data.create_guard_file(
            self.api_client.create_activation_guard(
                offline_data.decode_offline_activation()["request"]["request_id"]
            )
        )

        return self.licensefile_handler.create_request_file(
            req_data=offline_data, offline_path=req_path
        )

    def activate_bundle_offline(self, ls_activation_path: str) -> dict[str, License]:
        """
        Activate offline bundle licenses

        Args:
            ls_activation_path (str): path to a .lic file

        Returns:
            dict[str, License]: dictionary of licenses in a bundle
        """

        data = self.licensefile_handler.load_offline_response(ls_activation_path)

        if len(data._data) == 0:
            return

        response = self.api_client.activate_bundle_offline(data=data)
        bundles_response = self.licensefile_handler._cache.get_bundle_response(response)

        response_licenses = {}

        for product_code, license_response in bundles_response.items():
            license_storage = DataStorage(self._conf)
            license_storage._cache.product = product_code
            license_storage._cache.update_cache(
                "activate_offline_license_bundle", license_response
            )
            response_licenses[product_code] = License(
                product=product_code,
                api_client=self.api_client,
                licensefile_handler=license_storage,
            )
            license_obj = response_licenses[product_code]
            self.licensefile_handler.save_licensefile(
                license_obj.licensefile_handler._cache
            )

        self.cache_licenses = response_licenses

        return self.cache_licenses

    def deactivate_bundle_offline(
        self, license_id: LicenseID, offline_path: str, unique_license_id: int = None
    ) -> str:
        """
        Generates .req file for the offline deactivation

        Args:
            license_id (LicenseID): license_id
            offline_path (str): path of the .req file
            unique_license_id (int): unique license id

        Returns:
            str: path of the deactivation file
        """

        for license_obj in self.get_current_bundle().values():
            license_obj.licensefile_handler._cache.deactivate()
            self.licensefile_handler.save_licensefile(
                license_obj.licensefile_handler._cache
            )

        data = self.api_client.deactivate_offline_dump(
            product=self.licensefile_handler._cache.product,
            bundle_code=None,
            hardware_id=None,
            license_key=license_id.key,
            license_id=unique_license_id,
            username=license_id.username,
        )
        offline_data = OfflineActivation()
        offline_data.set_is_activation(False)
        offline_data.set_data(data)

        return self.licensefile_handler.create_request_file(offline_data, offline_path)

    def check_bundle(
        self,
        license_id: LicenseID,
        hardware_id: str = None,
        unique_license_id: int = None,
        include_expired_features: bool = False,
        env: str = None,
    ) -> dict[str, License]:
        """
        Check bundle and update the licensefile

        Args:
            license_id (LicenseID): license_id
            hardware_id (str, optional): A unique identifier for the hardware. Defaults to None.
            unique_license_id (int, optional): A unique identifier for the license. Defaults to None.
            include_expired_features (bool, optional): If True, includes expired license features in the check. Defaults to False.
            env (str, optional): optional param takes "win", "win32", "win64", "mac", "linux", "linux32" or "linux64". Defaults to None.

        Returns:
            dict[str, License]: A dictionary where the keys are product short codes and the values are License objects.
        """

        response = self.api_client.check_bundle(
            product=self._conf._product,
            hardware_id=hardware_id,
            license_key=license_id.key,
            license_id=unique_license_id,
            username=license_id.username,
            include_expired_features=include_expired_features,
            env=env,
        )

        cache_licenses = self.get_current_bundle()

        bundles_response = self.licensefile_handler._cache.get_bundle_response(response)

        for product_code, license_obj in cache_licenses.items():
            license_obj.licensefile_handler._cache.update_cache(
                "check_license", bundles_response[product_code]
            )
            self.licensefile_handler.save_licensefile(
                license_obj.licensefile_handler._cache
            )

        logging.info("Online bundle check completed successfully")
        return cache_licenses

    def deactivate_bundle(
        self,
        license_id: LicenseID,
        hardware_id: str = None,
        unique_license_id: int = None,
        remove_local_data: bool = False,
    ) -> None:
        """
        Deactivate bundle

        Args:
            license_id (LicenseID): license_id
            hardware_id (str, optional): hardware id. Defaults to None.
            unique_license_id (int, optional): A unique identifier for the license. Defaults to None.
            remove_local_data (bool, optional): remove licensefile from storage. Defaults to False.
        """

        self.api_client.deactivate_bundle(
            product=self._conf._product,
            hardware_id=hardware_id,
            license_key=license_id.key,
            license_id=unique_license_id,
            username=license_id.username,
        )

        if remove_local_data:
            self.licensefile_handler.delete_licensefile()
            self.cache_licenses = None
            return

        for license_obj in self.get_current_bundle().values():
            license_obj.licensefile_handler._cache.deactivate()
            self.licensefile_handler.save_licensefile(
                license_obj.licensefile_handler._cache
            )



import base64
import logging

from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature.pkcs1_15 import PKCS115_SigScheme
from OpenSSL import crypto


class SignatureVerifier:
    PUBLIC_KEY = """-----BEGIN PUBLIC KEY-----
    MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAwEt7ZQWSzeBYNBwyi3KW
    +XU/6I+ju90R5ZpTvSE4VWL8KSgVJ6bWKIhaKTL2hbUpIQQgS5ZKfa4SEEMdIm5k
    xe0u2F64JFAVQunx/1O/UXsD7ADVt93Q/hxA9Npa16tKepZoyhi79sxpaxyy/WTd
    sZKuTLApX7bWX6/XPhaPjNyiWeeK2Naka44B+F+PwN/ey3rZUra3pBltShwy2dmK
    IxJVmprf5ttMYBB+ouqPin3VgDw5Jq1FZwpLBiOe+ogR+sHu7QYWwLq6AgC4e3Jq
    gryEvqfJr/XkvOnAdcAIZm6tK6DdnLWCy6Onrk7t0VQK5nnGF1CKE+jcAByWopcI
    VCwfYrs1UCSY1YibKXNJoyvEhgLyCC+KAgXKf8omcg1Q18XKu5XSrvrCzXLnRnaF
    NpFDjcg5AJJab78hX7qGPC+e8PjuBYwh2vtx5mFj7/c+T59JM/vXwwvW9DsnDztD
    WEDFhzGanU71NgwrZbRoNSyTW0UjbZsyJVSIX6233Ng0y5L9mDe8p8P+u1B0sXAA
    ozEKL/yG4Qu5r4LIw4iv6JVPT1xbWlH4Vc4KaN3toaf4+G0EkFy6ncvBncWifLAR
    SoWpQ5YsygDdQNVYdmMsoQ76UTuNxo3eZ0sQJjbQVBqlkcXrAdtUCpuojPRsl/Xs
    9zVOJrzZPG0I98E5quNbRkMCAwEAAQ==
    -----END PUBLIC KEY-----"""

    def __init__(self):
        rsa_public_key = RSA.importKey(self.PUBLIC_KEY)
        self.verifier = PKCS115_SigScheme(rsa_public_key)

    def verify_signature(self, message, signature):
        hash = SHA256.new(message)
        self.verifier.verify(hash, signature)

    def verify_license_signature(
        self, hardware_id, auth_value, validity_period, signature_b64
    ):
        message = f'{hardware_id}#{auth_value}#{validity_period if validity_period else ""}'.lower().encode()
        signature = base64.b64decode(signature_b64)
        self.verify_signature(message, signature)

    def verify_license_signature_v2(self, message, signature_b64):
        signature = base64.b64decode(signature_b64)
        self.verify_signature(message, signature)


class FSSignatureVerifier(SignatureVerifier):

    def __init__(self, chain_path: str = None):
        if chain_path:
            self.leaf, self.intermediate, self.root = self.load_certs(chain_path)
            self.verify_chain(self.leaf, self.intermediate, self.root)

            self.verifier = self.load_verifier(self.leaf)

    def load_certs(self, chain_path: str) -> None:
        with open(chain_path, "rb") as file:
            pem_data = file.read()

        certs = []
        logging.info("Loading certificates ...")
        for cert in pem_data.split(b"-----END CERTIFICATE-----"):
            if b"-----BEGIN CERTIFICATE-----" in cert:
                cert += b"-----END CERTIFICATE-----\n"
                certs.append(crypto.load_certificate(crypto.FILETYPE_PEM, cert))

        return certs

    def verify_chain(self, leaf, intermediate, root):
        logging.info("Chain verification")
        store = crypto.X509Store()

        store.add_cert(root)
        store.add_cert(intermediate)

        store_ctx = crypto.X509StoreContext(store, leaf)
        store_ctx.verify_certificate()
        logging.info("Certificate chain is valid!")

    def load_verifier(self, leaf_cert):
        return PKCS115_SigScheme(
            RSA.importKey(
                crypto.dump_publickey(
                    crypto.FILETYPE_PEM, leaf_cert.get_pubkey()
                ).decode()
            )
        )
